%This case study compares the optimal decisions generated by portfolio
%optimization with Exponential, Logarithmic, and Linear-Quadratic utility functions.
%The shell version of this case study is described in the
%file CS_Optimization_Exponential_Log_and_Linear_Quadratic_Utilities.pdf
%which is located in ...\PSG\documents directory.
%MATLAB version of this case study is in help section "MATLAB Environment", 
%sub-section "Case Studies with PSG MATLAB",sub-section
%"CS_Optimization_Exponential_Log_and_Linear_Quadratic_Utilities".

clear;

% Load data
  load('Data');
  

% Pack matrices and points
    count = 1;
    iargstruc_arr(count) = tbpsg_matrix_pack('matrix_H', H, header_matrix);
    count = count + 1;
    iargstruc_arr(count) = tbpsg_matrix_pack('matrix_M', M, header_matrix);
    count = count + 1;
    iargstruc_arr(count) = tbpsg_matrix_pack('matrix_budget', budget, header_matrix);
    count = count + 1;
    iargstruc_arr(count) = tbpsg_point_pack('point_lowerbounds', lb, header_point);
    count = count + 1;
    iargstruc_arr(count) = tbpsg_point_pack('point_upperbounds', ub, header_point);
    count = count + 1;

% Solve the optimization problem 1 (CS.1) - (CS.3) with Exponential utility
%function

% Generate the first problem statement
    
    clear problem_statement1;
    problem_statement1 = sprintf('%s\n',...
    'maximize',...
    '  exp_eut(1,matrix_H)',...
    'Constraint: == 1',...
    '  linear_budget(matrix_budget)',...
    'Box: >= point_lowerbounds, <= point_upperbounds',...
    '');

% Optimize the first problem      
    [solution_str1, outargstruc_arr1] = tbpsg_run(problem_statement1, iargstruc_arr);

% Extract optimal solution of the first problem 
point_variables1 = tbpsg_optimal_point_vars(solution_str1, outargstruc_arr1);
Points1 = tbpsg_optimal_point_data(solution_str1, outargstruc_arr1);

% Get solving time for Problem 1
time1 = tbpsg_time(solution_str1, outargstruc_arr1);
data_loading_time_Problem_1 = time1(1);
preprocessing_time_Problem_1 = time1(2);
solving_time_Problem_1 = time1(3);

% Solve the optimization problem 2 (CS.4) - (CS.6) with Linear-Quadratic
% utility function

% Set parameters for the Linear-Quadratic utility function
     
    q = -20;
    u = 1;
    r = 10;
    s = 3;
    coef_PM2_1 =-(u+q)/(2*(r -s));

% Generate the second problem statement
    str2_1 = sprintf('maximize');
    str2_3 = sprintf('%.3f%s', u,'*avg_g_matrix_scenarios(matrix_H)');
    str2_4 = sprintf('%.4f%s%.3f%s',-coef_PM2_1,'*pm2_pen_matrix_scenarios_1(',-r,', matrix_H)');
    str2_5 = sprintf('%.4f%s%.3f%s',coef_PM2_1,'*pm2_pen_matrix_scenarios_2(',-s,', matrix_H)');
    str2_6 = sprintf('Constraint: == 1');
    str2_7 = sprintf('linear(matrix_budget)');
    str2_8 = sprintf('Box: >= point_lowerbounds, <= point_upperbounds');
    clear problem_statement2;
    
    problem_statement2 = sprintf('%s\n', str2_1, str2_3, str2_4, str2_5, str2_6, str2_7, str2_8);

% Optimize the second problem 
    [solution_str2, outargstruc_arr2] = tbpsg_run(problem_statement2, iargstruc_arr);    
    
% Extract optimal solution of the second problem 
point_variables2 = tbpsg_optimal_point_vars(solution_str2, outargstruc_arr2);
Points2 = tbpsg_optimal_point_data(solution_str2, outargstruc_arr2);

% Get solving time for Problem 2
time2 = tbpsg_time(solution_str2, outargstruc_arr2);
data_loading_time_Problem_2 = time2(1);
preprocessing_time_Problem_2 = time2(2);
solving_time_Problem_2 = time2(3);

% Solve the optimization problem 3 (CS.7) - (CS.9) with Logarithmic utility
% function
% Generate the third problem statement

    clear problem_statement3;
    problem_statement3 = sprintf('%s\n',...
    'maximize',...
    '  log_eut(matrix_M)',...
    'Constraint: == 1',...
    '  linear(matrix_budget)',...
    'Box: >= point_lowerbounds, <= point_upperbounds',...
    '');

% Optimize the third problem 
    [solution_str3, outargstruc_arr3] = tbpsg_run(problem_statement3, iargstruc_arr);
    
% Extract optimal solution of the third problem 
point_variables3 = tbpsg_optimal_point_vars(solution_str3, outargstruc_arr3);
Points3 = tbpsg_optimal_point_data(solution_str3, outargstruc_arr3);

% Get solving time for Problem 3
time3 = tbpsg_time(solution_str3, outargstruc_arr3);
data_loading_time_Problem_3 = time3(1);
preprocessing_time_Problem_3 = time3(2);
solving_time_Problem_3 = time3(3);    
    
% Display solving times the Problems 1 - 3
     
    fprintf('\n\nTiming for Problem 1');
    fprintf('\nLoading time: %f\nPreprocessing time: %f\nSolving time: %f\n',...
        data_loading_time_Problem_1, preprocessing_time_Problem_1,...
        solving_time_Problem_1);
     
    fprintf('\n\nTiming for Problem 2');
    fprintf('\nLoading time: %f\nPreprocessing time: %f\nSolving time: %f\n',...
        data_loading_time_Problem_2, preprocessing_time_Problem_2,...
        solving_time_Problem_2);
    
    fprintf('\n\nTiming for Problem 3');
    fprintf('\nLoading time: %f\nPreprocessing time: %f\nSolving time: %f\n',...
        data_loading_time_Problem_3, preprocessing_time_Problem_3,...
        solving_time_Problem_3);
    
% Display table comparing three solutions
    
    fprintf('\n%s','Components      Point Problem 1               Point Problem 2                 Point Problem 3');
    for j=1:size(Points1,2)
       fprintf('\n%s\t\t\t\t\t%.4f\t\t\t\t\t\t\t%.4f\t\t\t\t\t\t\t%.4f', point_variables2{j}, Points1(j), Points2(j), Points3(j));
    end
    fprintf('\n');
    
%Generate graph comparing three solutions
    Points =[Points1' Points2' Points3'];
    str_points = ['Exponential_Utility ' 'Linear-Quadratic_Utility ' 'Logarithmic_Utility '];
    leg = strread(str_points, '%s'); 
    leg = strrep(leg, '_', ' ');
    figure
    plot(Points,'.', 'MarkerSize',20)
    set(gca,'XTick',1:1:size(point_variables2,1));
    hAxes = gca;
    set(hAxes, 'XTickLabel', point_variables2);
    grid on;
    legend(leg, 'Location', 'West');
    xlabel('Instruments');
    ylabel('Portion of the Wealth Invested in Instrument ');
    title('Comparison of Solutions Obtained with Exponential, Logarithmic and Linear-Quadratic Utility');

%=======================================================================|
%American Optimal Decisions, Inc. Copyright                             |
%Copyright American Optimal Decisions, Inc. 2007-2016.                 |
%American Optimal Decisions (AOD) retains copyrights to this material.  |
%                                                                       |
%Permission to reproduce this document and to prepare derivative works  |
%from this document for internal use is granted, provided the copyright |
%and No Warranty  statements are included with all reproductions       |
%and derivative works.                                                  |
%                                                                       |
%For information regarding external or commercial use of copyrighted    |
%materials owned by AOD, contact AOD at support@aorda.com.              |
%=======================================================================|

    